--[[
Name: SliceWatcher
Revision: $Rev: 26468 $
Author(s): Kazun of Warsong (badhex@fusemail.com)
SVN: http://svn.wowace.com/root/trunk/SliceWatcher/
Description: Monitor the Slice and Dice buff.
Dependencies: AceLibrary, CompostLib, DewdropLib, TabletLib, Fubar
]]--
SLICEWATCHER = { }

local MAJOR_VERSION = "SliceWatcher"
local MINOR_VERSION = "$Revision: 26468 $"

local L = AceLibrary("AceLocale-2.2"):new("SliceWatcher")
SliceWatcher = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceConsole-2.0", "AceEvent-2.0")
local SliceWatcher = SliceWatcher
SliceWatcher:RegisterDB("SliceWatcherDB")

SLICEWATCHER.DBDEFAULT = {
	barWidth = 792,
	barHeight = 20,
	barFadeTo = 0.3,
	barLowGlow = true,
	barLock = false,
	barSound = true,
}

SLICEWATCHER.talent = 0
SLICEWATCHER.SnDStart = 0
SLICEWATCHER.SnDEnd = 0
SLICEWATCHER.maxBar = 0
SLICEWATCHER.Played = true

SLICEWATCHER.BASE_TIME = 6
SLICEWATCHER.STEP_TIME = 3
SLICEWATCHER.MULT = {0.0, 0.15, 0.30, 0.45}

L:RegisterTranslations("enUS", function() return {
	-- syntax
	["width"] = true,
	["Set the width of the bar"] = true,
	["Please include a width."] = true,
	["<width>"] = true,
	["height"] = true,
	["Set the height of the bar"] = true,
	["Please include a width."] = true,
	["<height>"] = true,
	["lock"] = true,
	["Lock or unlock the bar"] = true,
	["sound"] = true,
	["Turn the expire sound on or off"] = true,
	["lowglow"] = true,
	["Turn the low value glow feature on or off"] = true,
	["Low Glow ON."] = true,
	["Low Glow OFF."] = true,
	["Sound is now ON."] = true,
	["Sound is now OFF."] = true,
	["Bar is now locked."] = true,
	["Bar is now unlocked."] = true,
	["Improved Slice and Dice Rank"] = true,
	["Improved Slice and Dice"] = true,
	["fadeto"] = true,
	["Change the alpha value to fade out to."] = true,
} end)

L:RegisterTranslations("deDE", function() return {
	-- syntax
	["Improved Slice and Dice Rank"] = "Verbessertes Zerh\195\164ckseln Rang",
	["Improved Slice and Dice"] = "Verbessertes Zerh\195\164ckseln",
} end)

L:RegisterTranslations("koKR", function() return {
	-- syntax
	["width"] = "넓이",
	["Set the width of the bar"] = "바의 넓이를 설정합니다.",
	["Please include a width."] = "넓이를 설정하세요.",
	["<width>"] = "<넓이>",
	["height"] = "높이",
	["Set the height of the bar"] = "바의 높이를 설정합니다.",
	["Please include a width."] = "높이를 설정하세요.",
	["<height>"] = "<높이>",
	["lock"] = "잠금",
	["Lock or unlock the bar"] = "바를 고정시키거나 이동시킵니다.",
	["lowglow"] = "빛효과",
	["Turn the low value glow feature on or off"] = "바의 빛 효과를 켜거나 끔니다.",
	["Low Glow ON."] = "낮은 빛 효과 켬",
	["Low Glow OFF."] = "낮은 빛 효과 끔",
	["sound"] = "효과음",
	["Turn the expire sound on or off"] = "효과음을 알려줍니다.",
	["Sound is now ON."] = "효과음 켬",
	["Sound is now OFF."] = "효과음 끔",
	["Bar is now locked."] = "지금 바 고정",
	["Bar is now unlocked."] = "지금 바 이동",
	["Improved Slice and Dice Rank"] = "난도질 연마 레벨",
	["Improved Slice and Dice"] = "난도질 연마",
	["fadeto"] = "투명도",
	["Change the alpha value to fade out to."] = "점점 어두워질 투명도를 변경합니다.",
} end)

SliceWatcher:RegisterChatCommand({"/slice", "/sw", '/slicewatcher'}, {
	type = 'group',
	args = {
		width = {
			type = "range",
			name = L["width"],
			desc = L["Set the width of the bar"],
		    get = function()
		        return SliceWatcher.db.profile.barWidth
		    end,
		    set = "SetBarWidth",
		    min = 10,
		    max = 1000,
		    step = 1,
		    isPercent = false,
		},
		height = {
			type = "range",
			name = L["height"],
			desc = L["Set the height of the bar"],
		    get = function()
		        return SliceWatcher.db.profile.barHeight
		    end,
		    set = "SetBarHeight",
		    min = 2,
		    max = 100,
		    step = 1,
		    isPercent = false,
		},
		fadealpha = {
		    type = 'range',
		    name = L["fadeto"],
		    desc = L["Change the alpha value to fade out to."],
		    get = function()
		        return SliceWatcher.db.profile.barFadeTo
		    end,
		    set = function(v)
		        SliceWatcher.db.profile.barFadeTo = v
		    end,
		    min = 0,
		    max = 1, -- since percentage, max is 100%,
		    step = 0.05, -- every 5%
		    isPercent = true,
		},
		
        lock = {
            type = 'toggle',
            name = L["lock"],
            desc = L["Lock or unlock the bar"],
            get = function() return not SliceWatcher.db.profile.barLock end,
            set = "BarLock"
        },
		sound = {
			type = "toggle",
			name = L["sound"],
			desc = L["Turn the expire sound on or off"],
			get = function() return SliceWatcher.db.profile.barSound end,
			set = "BarSound",
		},
		lowglow = {
			type = "toggle",
			name = L["lowglow"],
			desc = L["Turn the low value glow feature on or off"],
			get = function() return SliceWatcher.db.profile.barLowGlow end,
			set = "LowGlow",
		},
	}
})

function SliceWatcher:LowGlow()
	self.db.profile.barLowGlow = not self.db.profile.barLowGlow
end

function SliceWatcher:BarSound()
	self.db.profile.barSound = not self.db.profile.barSound
end

function SliceWatcher:BarLock()
	self.db.profile.barLock = not self.db.profile.barLock
	SliceWatcherBar:EnableMouse(self.db.profile.barLock)
end

function SliceWatcher:SetBarWidth(width)
	SliceWatcher.db.profile.barWidth = tonumber(width)
	SliceWatcherBar:SetWidth(self.db.profile.barWidth+8)
	SliceWatcherBar1:SetWidth(self.db.profile.barWidth+8)
	SliceWatcherBar1StatusBar:SetWidth(self.db.profile.barWidth)
	SliceWatcherBar1StatusBarB:SetWidth(self.db.profile.barWidth)
	SliceWatcherBar1StatusBarB:SetWidth(self.db.profile.barWidth*3)
	SliceWatcherBar1StatusBarB:SetMinMaxValues(0, self.db.profile.barWidth)
	SliceWatcher:event_SetTalent(false)
end

function SliceWatcher:SetBarHeight(height)
	SliceWatcher.db.profile.barHeight = tonumber(height)
	SliceWatcherBar:SetHeight(self.db.profile.barHeight+8)
	SliceWatcherBar1:SetHeight(self.db.profile.barHeight+8)
	SliceWatcherBar1StatusBar:SetHeight(self.db.profile.barHeight)
	SliceWatcherBar1StatusBarB:SetHeight(self.db.profile.barHeight)
	SliceWatcherBar1StatusBarSpark:SetHeight(self.db.profile.barHeight*2)
	SliceWatcherBar1Glow:SetHeight(self.db.profile.barHeight*5)
end
		
function SliceWatcher:OnInitialize()
	-- stuff to do when the mod starts up
	self:RegisterDefaults('profile', SLICEWATCHER.DBDEFAULT )
	self:SetProfile("char")
end

function SliceWatcher:OnEnable()
	self:RegisterEvent("PLAYER_AURAS_CHANGED", "event_PlayerAurasChanged")
	self:RegisterEvent("CHARACTER_POINTS_CHANGED", "event_CharacterPointsChanged")
	self:RegisterEvent("PLAYER_COMBO_POINTS", "event_PlayerComboPoints")
	-- remove old combo points when you zone (ie. leave a battleground)
	self:RegisterEvent("ZONE_CHANGED", "event_PlayerComboPoints")
	self:ScheduleEvent("SliceWatcher", SliceWatcher.event_SetTalent, 1)
end

function SliceWatcher:event_SetTalent(firstRun)
	local self = SliceWatcher
	if firstRun == nil then firstRun = true end
	SLICEWATCHER.talent = self:util_TalentPoints()
	if not SLICEWATCHER.talent then SLICEWATCHER.talent = 0 end
	self:Print("SliceWatcher by Kazun of Warsong")
	self:Print("%s: %i (%i%%)", L["Improved Slice and Dice Rank"], SLICEWATCHER.talent, SLICEWATCHER.MULT[SLICEWATCHER.talent+1] * 100)
	
	SliceWatcherBar1:SetAlpha(1)
	SliceWatcherBar1:SetWidth(self.db.profile.barWidth+8)
	SliceWatcherBar1:SetHeight(self.db.profile.barHeight+8)
	SliceWatcherBar1:Show()
	
	SliceWatcherBar1StatusBar:SetAlpha(.75)
	SliceWatcherBar1StatusBar:SetMinMaxValues(0, 1)
	SliceWatcherBar1StatusBar:SetValue(0)
	SliceWatcherBar1StatusBar:SetWidth(self.db.profile.barWidth)
	SliceWatcherBar1StatusBar:SetHeight(self.db.profile.barHeight)
	
	SliceWatcherBar1StatusBarB:SetAlpha(1)
	SliceWatcherBar1StatusBarB:SetStatusBarColor(1, 1, 0, .9)
	SliceWatcherBar1StatusBarB:SetWidth(self.db.profile.barWidth)
	SliceWatcherBar1StatusBarB:SetHeight(self.db.profile.barHeight)
	SliceWatcherBar1StatusBarB:SetMinMaxValues(0, 1)
	SliceWatcherBar1StatusBarB:SetValue(0)
	SliceWatcherBar1StatusBarB:Show()

	SliceWatcherBar1StatusBarSpark:SetHeight(self.db.profile.barHeight*2)
	SliceWatcherBar1StatusBarSpark:Hide()
	
	SliceWatcherBar:SetWidth(self.db.profile.barWidth+8)
	SliceWatcherBar:SetHeight(self.db.profile.barHeight+8)

	SliceWatcherBar1Glow:SetHeight(self.db.profile.barHeight*5)
	SliceWatcherBar1Glow:SetWidth(self.db.profile.barWidth*3)
	SliceWatcherBar1Glow:SetVertexColor(1, 0, 0)
	
	SliceWatcherBar:EnableMouse(self.db.profile.barLock)
end

function SliceWatcher:event_PlayerComboPoints()
		-- sync our buff timer
		local self = SliceWatcher
		local buffTime = self:util_SnDBuffTime()
		if buffTime > 0 then
			SliceWatcherBar1:SetAlpha(1)
			SliceWatcherBar1StatusBarSpark:Show()
			SLICEWATCHER.SnDStart = GetTime();
			SLICEWATCHER.SnDEnd = GetTime() + buffTime;
			SLICEWATCHER.maxBar = GetTime() + (1.0 + SLICEWATCHER.MULT[SLICEWATCHER.talent+1]) * (6 + (3 * MAX_COMBO_POINTS))
			SliceWatcherBar1StatusBar:SetMinMaxValues(SLICEWATCHER.SnDStart, SLICEWATCHER.maxBar);
			SLICEWATCHER.Played = false
			SliceWatcherBar1Glow:SetAlpha(0)
			self:ScheduleEvent("SliceWatch", SliceWatcher.event_PlayerComboPoints, 2)
		end
		-- update our combo point bar
		if GetComboPoints() > 0 then
			local maxTime = ((1.0 + SLICEWATCHER.MULT[SLICEWATCHER.talent+1]) * (6 + (3 * MAX_COMBO_POINTS)))
			local barTime = ((1.0 + SLICEWATCHER.MULT[SLICEWATCHER.talent+1]) * (6 + (3 * GetComboPoints())))
			SliceWatcherBar1:SetAlpha(1)
			SliceWatcherBar1StatusBarB:SetMinMaxValues(0, self.db.profile.barWidth)
			SliceWatcherBar1StatusBarB:SetValue(barTime / maxTime * self.db.profile.barWidth);
		else
			SliceWatcherBar1StatusBarB:SetValue(0);
		end
end

function SliceWatcher:util_SnDBuffTime()
	for i=0,31 do
		local id,cancel = GetPlayerBuff(i,"HELPFUL|HARMFUL|PASSIVE");
		local buffText = GetPlayerBuffTexture(id)	
		if(id > 0 and buffText and string.find(buffText,"Ability_Rogue_SliceDice")) then
			local timeleft = GetPlayerBuffTimeLeft(id);
			return timeleft
		end
	end
	return 0
end

function SliceWatcher:xml_OnUpdate()
	-- every frame update the bar graphics so it appears smooth
	local status = GetTime()
	local self = SliceWatcher

	if ( status <= SLICEWATCHER.SnDEnd ) then
		local sparkPot = ((SLICEWATCHER.SnDEnd - status) / (SLICEWATCHER.maxBar - SLICEWATCHER.SnDStart)) * self.db.profile.barWidth;
		SliceWatcherBar1StatusBarSpark:SetPoint("CENTER", "SliceWatcherBar1StatusBar", "LEFT", sparkPot, 0);			
		SliceWatcherBar1StatusBar:SetMinMaxValues(SLICEWATCHER.SnDStart, SLICEWATCHER.maxBar);
		SliceWatcherBar1StatusBar:SetValue(SLICEWATCHER.SnDStart + SLICEWATCHER.SnDEnd - status);
		
		if GetComboPoints() ~= 0 then
			local maxTime = ((1.0 + SLICEWATCHER.MULT[SLICEWATCHER.talent+1]) * (6 + (3 * MAX_COMBO_POINTS)))
			local barTime = ((1.0 + SLICEWATCHER.MULT[SLICEWATCHER.talent+1]) * (6 + (3 * GetComboPoints())))
			SliceWatcherBar1StatusBarB:SetValue(barTime / maxTime * self.db.profile.barWidth);
		else
			SliceWatcherBar1StatusBarB:SetValue(0);
		end

		--DiscoDiceBar1Text:SetText(DiscoDice.SnDText)
	elseif (GetComboPoints() == 0) then
		local alpha = SliceWatcherBar1:GetAlpha() - 0.025;
		if( alpha > self.db.profile.barFadeTo) then
			SliceWatcherBar1:SetAlpha(alpha);
			SliceWatcherBar1Glow:SetAlpha(alpha)
			SliceWatcherBar1StatusBarSpark:SetAlpha(alpha);
			if SLICEWATCHER.Played == false and self.db.profile.barSound == true then
				PlaySoundFile("Interface\\AddOns\\SliceWatcher\\slice.wav")
				SLICEWATCHER.Played = true
				SliceWatcherBar1StatusBarSpark:Hide()
			end
		else
			SliceWatcherBar1StatusBarSpark:Hide()
			SliceWatcherBar1StatusBarSpark:SetAlpha(1)
			SliceWatcherBar1Glow:SetAlpha(0)
		end
	elseif ( SliceWatcherBar1:GetAlpha() > self.db.profile.barFadeTo) then
		if SLICEWATCHER.Played == false and self.db.profile.barSound == true then
			PlaySoundFile("Interface\\AddOns\\SliceWatcher\\slice.wav")
			SLICEWATCHER.Played = true
			SliceWatcherBar1StatusBarSpark:Hide()
		end
	end
	if (status <= SLICEWATCHER.SnDEnd and SLICEWATCHER.SnDEnd - status < 3 and self.db.profile.barLowGlow) then
		local galpha = SliceWatcherBar1Glow:GetAlpha() + 0.01
		if( galpha < 1) then
			SliceWatcherBar1Glow:SetAlpha(galpha)
		end
	end
end

function SliceWatcher:event_PlayerAurasChanged()
	SLICEWATCHER.hasBuff = self:util_SnDBuffTime() > 0
end

function SliceWatcher:event_CharacterPointsChanged()
	local oldTalent = SLICEWATCHER.talent
	SLICEWATCHER.talent = self:util_TalentPoints()
	if not SLICEWATCHER.talent then SLICEWATCHER.talent = 0 end
	if (SLICEWATCHER.talent ~= oldTalent) then
		self:Print("Improved Slice and Dice Rank: %i (%i%%)", SLICEWATCHER.talent, SLICEWATCHER.MULT[SLICEWATCHER.talent+1] * 100)
	end
end

-- retuns the talent points spent in slice and dice
function SliceWatcher:util_TalentPoints()
	local numTabs = GetNumTalentTabs()
	for t=1, numTabs do
	    local numTalents = GetNumTalents(t);
	    for i=1, numTalents do
	        local nameTalent, _, _, _, currRank, _ = GetTalentInfo(t,i);
	        if nameTalent == L["Improved Slice and Dice"] then
	        	return currRank
	        end
	    end
	end
	return 0
end
